# coding: utf-8

"""
    Hatchet API

    The Hatchet API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class TenantResource(str, Enum):
    """
    TenantResource
    """

    """
    allowed enum values
    """
    WORKER = "WORKER"
    WORKER_SLOT = "WORKER_SLOT"
    EVENT = "EVENT"
    TASK_RUN = "TASK_RUN"
    CRON = "CRON"
    SCHEDULE = "SCHEDULE"
    INCOMING_WEBHOOK = "INCOMING_WEBHOOK"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of TenantResource from a JSON string"""
        return cls(json.loads(json_str))
