# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: v1/shared/condition.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'v1/shared/condition.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19v1/shared/condition.proto\x12\x02v1\"t\n\x12\x42\x61seMatchCondition\x12\x19\n\x11readable_data_key\x18\x01 \x01(\t\x12\x1a\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32\n.v1.Action\x12\x13\n\x0bor_group_id\x18\x03 \x01(\t\x12\x12\n\nexpression\x18\x04 \x01(\t\"`\n\x1cParentOverrideMatchCondition\x12$\n\x04\x62\x61se\x18\x01 \x01(\x0b\x32\x16.v1.BaseMatchCondition\x12\x1a\n\x12parent_readable_id\x18\x02 \x01(\t\"N\n\x13SleepMatchCondition\x12$\n\x04\x62\x61se\x18\x01 \x01(\x0b\x32\x16.v1.BaseMatchCondition\x12\x11\n\tsleep_for\x18\x02 \x01(\t\"W\n\x17UserEventMatchCondition\x12$\n\x04\x62\x61se\x18\x01 \x01(\x0b\x32\x16.v1.BaseMatchCondition\x12\x16\n\x0euser_event_key\x18\x02 \x01(\t\"\xc5\x01\n\x0eTaskConditions\x12\x44\n\x1aparent_override_conditions\x18\x01 \x03(\x0b\x32 .v1.ParentOverrideMatchCondition\x12\x31\n\x10sleep_conditions\x18\x02 \x03(\x0b\x32\x17.v1.SleepMatchCondition\x12:\n\x15user_event_conditions\x18\x03 \x03(\x0b\x32\x1b.v1.UserEventMatchCondition\"\x8f\x01\n\x1e\x44urableEventListenerConditions\x12\x31\n\x10sleep_conditions\x18\x01 \x03(\x0b\x32\x17.v1.SleepMatchCondition\x12:\n\x15user_event_conditions\x18\x02 \x03(\x0b\x32\x1b.v1.UserEventMatchCondition*5\n\x06\x41\x63tion\x12\n\n\x06\x43REATE\x10\x00\x12\t\n\x05QUEUE\x10\x01\x12\n\n\x06\x43\x41NCEL\x10\x02\x12\x08\n\x04SKIP\x10\x03\x42\x42Z@github.com/hatchet-dev/hatchet/internal/services/shared/proto/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'v1.shared.condition_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z@github.com/hatchet-dev/hatchet/internal/services/shared/proto/v1'
  _globals['_ACTION']._serialized_start=764
  _globals['_ACTION']._serialized_end=817
  _globals['_BASEMATCHCONDITION']._serialized_start=33
  _globals['_BASEMATCHCONDITION']._serialized_end=149
  _globals['_PARENTOVERRIDEMATCHCONDITION']._serialized_start=151
  _globals['_PARENTOVERRIDEMATCHCONDITION']._serialized_end=247
  _globals['_SLEEPMATCHCONDITION']._serialized_start=249
  _globals['_SLEEPMATCHCONDITION']._serialized_end=327
  _globals['_USEREVENTMATCHCONDITION']._serialized_start=329
  _globals['_USEREVENTMATCHCONDITION']._serialized_end=416
  _globals['_TASKCONDITIONS']._serialized_start=419
  _globals['_TASKCONDITIONS']._serialized_end=616
  _globals['_DURABLEEVENTLISTENERCONDITIONS']._serialized_start=619
  _globals['_DURABLEEVENTLISTENERCONDITIONS']._serialized_end=762
# @@protoc_insertion_point(module_scope)
